/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.verknuepfung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse speichert die Daten der Attributgruppe !atg.sswSbaUeVerknpfung!.<p>
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2015/09/10 07:13:16 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class ParameterVerknuepfung
{
	/**
	 * Pid der Attributgruppe.
	 */
	private String m_pidAtg = "atg.sswSbaUeVerknpfung";
	/**
	 * Liste der Inputdefinitionen.
	 */
	private List<Input> m_input = new ArrayList<Input>();
	/**
	 * Reaktion der Ursacheneinheit Ein-/Ausschaltung.
	 */
	private LogischeVerknuepfung m_logischeVerknuepfung;
	
	
	public enum LogischeVerknuepfung
	{
		UND,
		ODER;
		
		public static LogischeVerknuepfung getWithDavWert(int wertDav)
		{
			if (wertDav == 0)
			{
				return UND;				
			}
			else if (wertDav == 1)
			{
				return ODER;
			}
			
			return UND;
		}

		public static String getText(LogischeVerknuepfung wert)
		{
			if (wert.equals(UND))
			{
				return "UND";				
			}
			else if (wert.equals(ODER))
			{
				return "ODER";
			}
			
			return "UND";
		}
	}
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public ParameterVerknuepfung()
	{
	}
	/**
	 * Konstruktor mit bernahme des bergebenen Datensatzes in die interne Struktur.
	 *  
	 * @param rdata Datensatz der Parameter.
	 */
	public ParameterVerknuepfung(ResultData rdata)
	{
		setdaten(rdata);
	}
	
	/**
	 * bernahme des bergebenen Datensatzes in die interne Struktur.
	 * 
	 * @param rdata Datensatz der Parameter.
	 */
	public void setdaten(ResultData rdata)
	{
		if (rdata != null)
		{
			m_input.clear();
			m_logischeVerknuepfung = LogischeVerknuepfung.UND;
			
			Data data = rdata.getData();
			if (data != null)
			{
				for (int i = 0; i < data.getArray("Input").getLength(); i++)
				{
					Data element = data.getArray("Input").getItem(i);

					addInput(element.getReferenceValue("Ursacheneinheit").getSystemObject(), element.getScaledValue("InputInvertiert").getValueText().equals("Ja") ? true : false);
				}
				
				m_logischeVerknuepfung = LogischeVerknuepfung.getWithDavWert(data.getUnscaledValue("LogischeVerknpfung").intValue());
			}
		}		
	}

	/**
	 * Erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeHarmoGeschwDiff</code> mit
	 * den Daten der internen Struktur.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @return erzeugter Datensatz.
	 */
	public Data erzeugeDatensatz(ClientDavConnection connection)
	{
		DataModel dm = connection.getDataModel();		
		Data dataNeu = connection.createData(dm.getAttributeGroup(m_pidAtg));

		if (dataNeu != null)
		{
			dataNeu.getArray("Input").setLength(0);
			
			if (m_input != null)
			{
				for (int i = 0; i < m_input.size(); i++)
				{
					Input input = m_input.get(i);

					dataNeu.getArray("Input").setLength(i+1);
					Data element = dataNeu.getArray("Input").getItem(i);
					
					element.getReferenceValue("Ursacheneinheit").setSystemObject(input.getUrsacheneinheit());			
					element.getScaledValue("InputInvertiert").setText(input.isInputInvertiert() ? "Ja" : "Nein");								
				}
			}

			dataNeu.getUnscaledValue("LogischeVerknpfung").set(getLogischeVerknpfung().ordinal());
			
			return dataNeu;
		}
		
		return null;
	}
	
	/**
	 * Gibt den Wert der Variable m_input zurck.
	 *
	 * @return Wert der Variable m_input.
	 */
	public List<Input> getInput()
	{
		return m_input;
	}

	/**
	 * Gibt den Wert der Variable m_input zurck.
	 *
	 * @return Wert der Variable m_input.
	 */
	public Input getInput(SystemObject ursacheneinheit)
	{
		if (m_input != null)
		{
			for (int i = 0; i < m_input.size(); i++)
			{
				Input input = m_input.get(i);
		
				if (input.getUrsacheneinheit().equals(ursacheneinheit))
				{
					return input;
				}
			}
		}
		
		return null;
	}
	/**
	 * Setzt den Wert der Variable m_input.
	 *
	 * @param input zu setzender Wert der Variable m_input.
	 */
	public void setInput(List<Input> input)
	{
		m_input = input;
	}
	
	/**
	 * Fgt der Liste der Inputdefinitionen eine Definition mit den gegebenen Werten hinzu.
	 * 
	 * @param ursacheneinheit Referenz auf eine Ursacheneinheit.
	 * @param inputInvertiert Kennung Input invertiert.
	 */
	public void addInput(SystemObject ursacheneinheit, boolean inputInvertiert)
	{
		if (m_input == null)
		{
			m_input = new ArrayList<ParameterVerknuepfung.Input>();
		}
		
		m_input.add(new Input(ursacheneinheit, inputInvertiert));
	}
	
	/**
	 * Gibt den Wert der Variable m_reaktion zurck.
	 *
	 * @return Wert der Variable m_reaktion.
	 */
	public LogischeVerknuepfung getLogischeVerknpfung()
	{
		return m_logischeVerknuepfung;
	}
	/**
	 * Setzt den Wert der Variable m_reaktion.
	 *
	 * @param reaktion zu setzender Wert der Variable m_reaktion.
	 */
	public void setLogischeVerknuepfung(LogischeVerknuepfung reaktion)
	{
		m_logischeVerknuepfung = reaktion;
	}	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "ParameterVerknuepfung [m_input=" + m_input + ", m_logischeVerknuepfung=" + m_logischeVerknuepfung + "]";
	}


	/**
	 * 
	 * Innere Klasse zum Speichern einer Inputdefinition. 
	 *
	 * @version $Revision: 1.2 $ / $Date: 2015/09/10 07:13:16 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	public class Input
	{
		/**
		 * Referenz auf ein Systemobjekt Ursacheneinheit.
		 */
		private SystemObject m_ursacheneinheit;
		/**
		 * Kennung Inputsignal invertiert.
		 */
		private boolean m_inputInvertiert;
		
		/**
		 * Default-Konstruktor der Klasse.
		 */
		public Input()
		{
			
		}

		/**
		 * Konstruktor der Klasse mit bernahme der Parameter.
		 * 
		 * @param ursacheneinheit Referenz auf eine Ursacheneinheit.
		 * @param inputInvertiert Kennung Input invertiert.
		 */
		public Input(SystemObject ursacheneinheit, boolean inputInvertiert)
		{
			m_ursacheneinheit = ursacheneinheit;
			m_inputInvertiert = inputInvertiert;
		}
		
		/**
		 * Gibt den Wert der Variable m_ursacheneinheit zurck.
		 *
		 * @return Wert der Variable m_ursacheneinheit.
		 */
		public SystemObject getUrsacheneinheit()
		{
			return m_ursacheneinheit;
		}

		/**
		 * Setzt den Wert der Variable m_ursacheneinheit.
		 *
		 * @param ursacheneinheit zu setzender Wert der Variable m_ursacheneinheit.
		 */
		public void setUrsacheneinheit(SystemObject ursacheneinheit)
		{
			m_ursacheneinheit = ursacheneinheit;
		}

		/**
		 * Gibt den Wert der Variable m_inputInvertiert zurck.
		 *
		 * @return Wert der Variable m_inputInvertiert.
		 */
		public boolean isInputInvertiert()
		{
			return m_inputInvertiert;
		}

		/**
		 * Setzt den Wert der Variable m_inputInvertiert.
		 *
		 * @param inputInvertiert zu setzender Wert der Variable m_inputInvertiert.
		 */
		public void setInputInvertiert(boolean inputInvertiert)
		{
			m_inputInvertiert = inputInvertiert;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString()
		{
			return "Input [m_ursacheneinheit=" + m_ursacheneinheit + ", m_inputInvertiert=" + m_inputInvertiert + "]";
		}		
	}
}